@@ -6,6 +6,7 @@ import copy
 import os
 from collections import OrderedDict, defaultdict
 from dataclasses import dataclass
+from multiprocessing import Pool
 from pathlib import Path
 from typing import Dict, List
 
@@ -790,7 +791,7 @@ class Coco:
         self.images.append(image)
 
     @classmethod
-    def from_coco_dict_or_path(cls, coco_dict_or_path, desired_name2id=None):
+    def from_coco_dict_or_path(cls, coco_dict_or_path, desired_name2id=None, remapping_dict=None):
         """
         Creates coco object from COCO formatted dict or COCO dataset file path.
 
@@ -800,13 +801,15 @@ class Coco:
                 List of COCO formatted dict or COCO dataset file path
             desired_name2id : dict
                 {"human": 1, "car": 2, "big_vehicle": 3}
+            remapping_dict: dict
+                {1:0, 2:1} maps category id 1 to 0 and category id 2 to 1
 
         Properties:
             images: list of CocoImage
             category_mapping: dict
         """
         # init coco object
-        coco = cls()
+        coco = cls(remapping_dict=remapping_dict)
 
         if type(coco_dict_or_path) == list:  # merge coco datasets if given as list
             # create coco_dict_list
@@ -831,7 +834,7 @@ class Coco:
 
         # arrange image id to annotation id mapping
         coco.add_categories_from_coco_category_list(coco_dict["categories"])
-        imageid2annotationlist = get_imageid2annotationlist_mapping(coco_dict)
+        imageid2annotationlist = get_imageid2annotationlist_mapping_mt(coco_dict)
         category_mapping = coco.category_mapping
 
         coco_image_list = []
@@ -839,7 +842,16 @@ class Coco:
             coco_image = CocoImage.from_coco_image_dict(coco_image_dict)
             annotation_list = imageid2annotationlist[coco_image_dict["id"]]
             for coco_annotation_dict in annotation_list:
-                category_name = category_mapping[coco_annotation_dict["category_id"]]
+                # apply category remapping if remapping_dict is provided
+                if coco.remapping_dict is not None:
+                    # apply category remapping (id:id)
+                    category_id = coco.remapping_dict[coco_annotation_dict["category_id"]] 
+                    # update category id
+                    coco_annotation_dict["category_id"] = category_id 
+                else:
+                    category_id = coco_annotation_dict["category_id"]
+                # get category name (id:name)
+                category_name = category_mapping[category_id] 
                 coco_annotation = CocoAnnotation.from_coco_annotation_dict(
                     category_name=category_name, annotation_dict=coco_annotation_dict
                 )
@@ -865,7 +877,7 @@ class Coco:
 
     @property
     def imageid2annotationlist(self):
-        return get_imageid2annotationlist_mapping(self.json)
+        return get_imageid2annotationlist_mapping_mt(self.json)
 
     @property
     def json(self):
@@ -1013,8 +1025,8 @@ class Coco:
 
         # create yolov5 data yaml
         data = {
-            "train": train_dir,
-            "val": val_dir,
+            "train": str(train_dir),
+            "val": str(val_dir),
             "nc": len(self.category_mapping),
             "names": list(self.category_mapping.values()),
         }
@@ -1306,9 +1318,32 @@ def merge_from_file(coco_path1: str, coco_path2: str, save_path: str):
     save_json(merged_coco_dict, save_path)
 
 
+def get_image_annotations(image_id, annotations):
+    image_annotations = []
+    for annotation in annotations:
+        if annotation["image_id"] == image_id:
+            image_annotations.append(annotation)
+    return image_id, image_annotations
+
+
+def get_imageid2annotationlist_mapping_mt(coco_dict):
+    """
+    Multithreaded version of sahi.utils.coco.get_imageid2annotationlist_mapping. (3 times faster)
+    """
+    imageid2annotationlist_mapping = {}
+    annotations = coco_dict["annotations"]
+    print("indexing coco dataset annotations...")
+    with Pool(processes=48) as pool:
+        args = [(image["id"], annotations) for image in coco_dict["images"]]
+        imageid_annotationlist_pairs = pool.starmap(get_image_annotations, tqdm(args, total=len(args)))
+    imageid2annotationlist_mapping = dict(imageid_annotationlist_pairs)
+
+    return imageid2annotationlist_mapping
+
+
 def get_imageid2annotationlist_mapping(coco_dict: dict) -> dict:
     """
-    Slice a large image into smaller windows.
+    Get image_id to annotationlist mapping for faster indexing.
 
     Arguments
     ---------
@@ -1334,7 +1369,8 @@ def get_imageid2annotationlist_mapping(coco_dict: dict) -> dict:
         }
     """
     imageid2annotationlist_mapping = {}
-    for image in coco_dict["images"]:
+    print("indexing coco dataset annotations...")
+    for image in tqdm(coco_dict["images"]):
         image_id = image["id"]
         imageid2annotationlist_mapping[image_id] = []
 
@@ -1478,6 +1514,7 @@ def split_coco_as_train_val(
     # divide annotations
     train_annotations = list()
     val_annotations = list()
+    print("splitting coco dataset into train/val...")
     for annotation in tqdm(coco_dict["annotations"]):
         image_index_for_annotation = image_id_2_idx[annotation["image_id"]]
         if image_index_for_annotation in train_indices:
